package at.jku.ssw.queue.test;

import java.util.Arrays;
import java.util.Comparator;
import java.util.NoSuchElementException;

import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

import at.jku.ssw.Iterator;
import at.jku.ssw.queue.PriorityQueue;
import at.jku.students.heap.ArrayPriorityQueue;

@SuppressWarnings("rawtypes")
public class PriorityQueueTest {

	private static final String[] VALUES = { "x", "t", "o", "g", "s", "m", "n", "a", "e", "r", "i", "a", "o", "t", "o",
			"a" };
	@Rule
	public ExpectedException thrown = ExpectedException.none();

	@Test
	public void testPushFail() {
		PriorityQueue pq = createPriorityQueue();
		thrown.expect(IllegalArgumentException.class);
		pq.push(null);
	}

	@Test
	public void testTakeFail() {
		PriorityQueue pq = new ArrayPriorityQueue(String.CASE_INSENSITIVE_ORDER);
		thrown.expect(NoSuchElementException.class);
		pq.take();
	}

	@Test
	public void testPeek() {
		PriorityQueue pq = createPriorityQueue();
		Assert.assertEquals("x", pq.peek());
	}

	@Test
	public void testPeekFail() {
		PriorityQueue pq = new ArrayPriorityQueue(String.CASE_INSENSITIVE_ORDER);
		thrown.expect(NoSuchElementException.class);
		pq.peek();
	}

	@Test
	public void testContains() {
		PriorityQueue pq = createPriorityQueue();
		Assert.assertTrue(pq.contains("x"));
		Assert.assertFalse(pq.contains("y"));
		Assert.assertTrue(pq.contains("a"));
		Assert.assertFalse(pq.contains("b"));
	}

	@Test
	public void testContainsFail() {
		PriorityQueue pq = createPriorityQueue();
		thrown.expect(IllegalArgumentException.class);
		pq.contains(null);
	}

	@Test
	public void testRemove() {
		PriorityQueue pq = createPriorityQueue();
		Assert.assertTrue(pq.remove("x"));
		Assert.assertFalse(pq.remove("y"));
		Assert.assertTrue(pq.remove("a"));
		Assert.assertTrue(pq.remove("a"));
		Assert.assertTrue(pq.remove("a"));
		Assert.assertFalse(pq.remove("a"));
		Assert.assertFalse(pq.remove("b"));
	}

	@Test
	public void testRemove2() {
		PriorityQueue pq = createPriorityQueue(String.CASE_INSENSITIVE_ORDER, "8", "2", "7", "0", "1", "5", "6");
		Assert.assertTrue(pq.remove("5"));
	}

	@Test
	public void testRemoveFail() {
		PriorityQueue pq = createPriorityQueue();
		thrown.expect(IllegalArgumentException.class);
		pq.remove(null);
	}

	@Test
	public void testIterator() {
		String[] values = VALUES.clone();
		PriorityQueue pq = createPriorityQueue(String.CASE_INSENSITIVE_ORDER, values);
		Iterator it = pq.iterator();
		Arrays.sort(values, String.CASE_INSENSITIVE_ORDER);
		int i = values.length;
		while (it.hasNext()) {
			Assert.assertEquals(values[--i], it.next());
		}
		Assert.assertEquals(0, i);
		thrown.expect(NoSuchElementException.class);
		it.next();
	}

	private static PriorityQueue createPriorityQueue() {
		return createPriorityQueue(String.CASE_INSENSITIVE_ORDER, VALUES);
	}

	private static <T> PriorityQueue createPriorityQueue(Comparator cmp, @SuppressWarnings("unchecked") T... values) {
		PriorityQueue pq = new ArrayPriorityQueue(cmp);
		for (int i = 0; i < values.length; i++) {
			pq.push(values[i]);
		}
		return pq;
	}

}